"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DomainEventEmitter = void 0;
const node_core_1 = require("@lokalise/node-core");
const HandlerSpy_1 = require("../queues/HandlerSpy");
class DomainEventEmitter {
    eventRegistry;
    eventHandlerMap = {};
    anyHandlers = [];
    metadataFiller;
    _handlerSpy;
    constructor({ eventRegistry, metadataFiller, }, options = {}) {
        this.eventRegistry = eventRegistry;
        this.metadataFiller = metadataFiller;
        this._handlerSpy =
            (0, HandlerSpy_1.resolveHandlerSpy)(options);
    }
    get handlerSpy() {
        if (!this._handlerSpy) {
            throw new Error('HandlerSpy was not instantiated, please pass `handlerSpy` parameter during queue service creation.');
        }
        return this._handlerSpy;
    }
    async emit(supportedEvent, data, metadata) {
        if (!data.timestamp) {
            data.timestamp = this.metadataFiller.produceTimestamp();
        }
        if (!data.id) {
            data.id = this.metadataFiller.produceId();
        }
        const eventTypeName = supportedEvent.publisherSchema.shape.type.value;
        if (!this.eventRegistry.isSupportedEvent(eventTypeName)) {
            throw new node_core_1.InternalError({
                errorCode: 'UNKNOWN_EVENT',
                message: `Unknown event ${eventTypeName}`,
            });
        }
        const eventHandlers = this.eventHandlerMap[eventTypeName];
        // No relevant handlers are registered, we can stop processing
        if (!eventHandlers && this.anyHandlers.length === 0) {
            // @ts-ignore
            return data;
        }
        const validatedEvent = this.eventRegistry
            .getEventDefinitionByTypeName(eventTypeName)
            .consumerSchema.parse({
            type: eventTypeName,
            ...data,
        });
        if (eventHandlers) {
            for (const handler of eventHandlers) {
                await handler.handleEvent(validatedEvent, metadata);
            }
        }
        for (const handler of this.anyHandlers) {
            await handler.handleEvent(validatedEvent, metadata);
        }
        if (this._handlerSpy) {
            this._handlerSpy.addProcessedMessage({
                // @ts-ignore
                message: {
                    ...validatedEvent,
                    ...(metadata !== undefined ? { metadata } : {}),
                },
                processingResult: 'consumed',
            }, validatedEvent.id);
        }
        // @ts-ignore
        return validatedEvent;
    }
    /**
     * Register handler for a specific event
     */
    on(eventTypeName, handler) {
        this.addOnHandler(eventTypeName, handler);
    }
    /**
     * Register handler for multiple events
     */
    onMany(eventTypeNames, handler) {
        for (const eventTypeName of eventTypeNames) {
            this.on(eventTypeName, handler);
        }
    }
    /**
     * Register handler for all events supported by the emitter
     */
    onAny(handler) {
        this.anyHandlers.push(handler);
    }
    addOnHandler(eventTypeName, handler) {
        if (!this.eventHandlerMap[eventTypeName]) {
            this.eventHandlerMap[eventTypeName] = [];
        }
        this.eventHandlerMap[eventTypeName].push(handler);
    }
}
exports.DomainEventEmitter = DomainEventEmitter;
//# sourceMappingURL=DomainEventEmitter.js.map